#!/usr/bin/env bash

# This is the master of model one training
# Parameters needed:
echo "--------------------------------------------------------"
echo "Usage of masters"
echo "hmmMaster <template> <watchdir> <collectdir> <logdir> <output> <output_a> <output_h> <round> <tmp> <childlist> <seed>"
echo "1. <template> The template file for the training"
echo "2. <watchdir> The directory to watch the output of childs"
echo "3. <collectdir> The directory to collect output of childs"
echo "4. <logdir> The directory to collect log of childs"
echo "5. <output> The output t table "
echo "6. <output_a> The output alignment table "
echo "7. <output_h> The output HTable"
echo "8. <round> The total number of iterations"
echo "9. <tmp> the temporary dir (Child process should be able to access)"
echo "10. <childlist> is the list of childs"
echo "11. <seed> is the previous t model"
echo "--------------------------------------------------------"
echo 

if [ -e $PWD/defines.inc ];
then
      source $PWD/defines.inc
else
	MODEL_ONE_NORM=/SMT/giza/view_source/newgiza/src/para/renomalizeModel1
	ATABLE_EXEC=/SMT/giza/view_source/newgiza/src/para/renomalizeAModel
	CHMM_EXEC=/SMT/giza/view_source/newgiza/src/para/combine_hmm
fi

if [ $# -lt "11" ];
then
	echo " NO! I need your command!"
	exit 1
fi

TEMPL=$1
WATCHDIR=$2
COLLDIR=$3
LOGDIR=$4
OUTPUT=$5
OUTPUT_A=$6
OUTPUT_H=$7
RUNS=${8}
TMPDIR=${9}
CHILDLIST=${10}
SEEDMODEL=${11}

echo "... Checking template" ${TEMPL}
if [ -r ${TEMPL} ]; # is template exists?
then
	echo " OK"
else
	echo " NO! The template cannot be read! "
	exit 1
fi

echo "... Checking watching dir" ${WATCHDIR}
if [ -d ${WATCHDIR} ]; # is watch dir exists
then
	echo " OK cleaning"
	rm -f ${WATCHDIR}/*
else
	echo " NO! The watching dir cannot be accessed"
	exit 1
fi

echo "... Checking collecting dir" ${COLLDIR}
if [ -d ${COLLDIR} ]; # is collection OK?
then
	echo " OK"
else
	echo " NO! The collecting dir cannot be accessed"
	exit 1
fi

echo "... Checking log dir" ${LOGDIR}
if [ -d ${LOGDIR} ]; # is log OK?
then
	echo " OK"
else
	echo " NO! The log dir cannot be accessed"
	exit 1
fi

echo "... Checking temporary dir" ${TMPDIR}
if [ -d ${TMPDIR} ]; # is tmp OK?
then
	echo " OK"
else
	echo " NO! The temporary dir cannot be accessed"
	exit 1
fi

echo "... Checking child list" ${CHILDLIST}
if [ -r ${CHILDLIST} ]; # is childlist OK?
then
	echo " OK"
else
	echo " NO! The child list cannot be accessed"
	exit 1
fi

echo "... Checking seed t model" ${SEEDMODEL}
if [ -r ${SEEDMODEL} ]; # is t model OK
then
	echo " OK"
else
	echo " NO! The previous t model cannot be accessed"
	exit 1
fi

LOGFILE=${LOGDIR}/hmm_master.log.$$

echo "If I die, check " $LOGFILE

touch ${OUTPUT}

if [ -w ${OUTPUT} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT}
	echo " OK"
else
	echo " NO! I will not be able to write output file"
	exit 1
fi
rm -f ${OUTPUT}



touch ${OUTPUT_A}

if [ -w ${OUTPUT_A} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT_A}
	echo " OK"
else
	echo " NO! I will not be able to write output file"
	exit 1
fi
rm -f ${OUTPUT_A}

touch ${OUTPUT_H}

if [ -w ${OUTPUT_H} ]; #is output writable?
then
	echo "... Checking access to output file" ${OUTPUT_H}
	echo " OK"
else
	echo " NO! I will not be able to write output file"
	exit 1
fi
rm -f ${OUTPUT_H}


if [ ${RUNS} -lt "1" ];
then
	echo " At least one round of training should be taken "
	exit 1
fi

ROUNDS=${RUNS}
TOTALROUND=${RUNS}
((ROUNDS = ROUNDS + 1))

echo "We will do " $TOTALROUND " (minus one) rounds of training"


# If we survives here, let's start running

CUR=1 # Current round
#Total number of childs

TOTAL_CHILD=`wc -l $CHILDLIST | awk '{print $1}' `

echo "Total number of children : " $TOTAL_CHILD

((TOTAL_CHILD = TOTAL_CHILD + 1))
CN=1
while [ $CN -lt $TOTAL_CHILD ];
do
	host[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $1}'`
	exe[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $2}'`
	conf[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $3}'`
	(( CN = CN + 1 ))
done


while [ $CUR -lt $ROUNDS ];
do
	echo "Start running round " $CUR
	if [ $CUR -gt "1" ]; # not first round,
	then
		(( PP = CUR - 1 ))
		prev=${TMPDIR}/HMM_R_${PP}_TTABLE.tbl
	else
		prev=$SEEDMODEL
	fi
	
	if [ $CUR -gt "1" ]; # not first round,
	then
		(( PP = CUR - 1 ))
		prev_a=${TMPDIR}/HMM_R_${PP}_ATABLE.tbl
		prev_h=${TMPDIR}/HMM_R_${PP}_HTABLE.tbl
	fi
	
	rm -f $2/*
	#################################################
	# Start all the process
	#################################################
	CN=1
	while [ $CN -lt $TOTAL_CHILD ];
	do
		output[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.diff
		acount[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.acount
		hcount[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.hmm
		signal[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish
		start[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish.starting
		fail[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish.failed
		log[${CN}]=$LOGDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.log
	        
		rm -f ${CN}.tmp
		echo "cd" $PWD > ${CN}.tmp
		echo ${exe[${CN}]} ${conf[${CN}]} ${output[${CN}]} ${acount[${CN}]} ${hcount[${CN}]} ${CUR} ${signal[${CN}]} ${log[${CN}]} $prev ${prev_a}  ${prev_h} >> ${CN}.tmp
		command[${CN}]=`echo ssh ${host[$CN]} source $PWD/${CN}.tmp`
#                                                                             
            touch  ${start[${CN}]}	
            echo ${command[${CN}]}
            ${command[${CN}]} >>${LOGFILE}&
            echo Child ${CN} Started on ${host[$CN]}
            (( CN = CN + 1 ))
	done
	sleep 1
	################################################
	# Watch what's going on
	################################################
	F=0
	while [ $F = "0" ]
	do
		sleep 10
		CN=1
		F=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			if [ -e ${start[${CN}]} ]; # Starting process failed
			then
				echo Starting child $CN failed!
				echo  ${host[${CN}]}  ${exe[${CN}]}  ${conf[${CN}]}
				exit 1
			fi	
			if [ -e ${signal[${CN}]} ]; # Finished
			then
				echo Child $CN on ${host[${CN}]} completed
			else
				if [ -e ${fail[${CN}]} ];
				then
					echo Child $CN on ${host[${CN}]} FAILED!
					echo Stop process and check ${log[${CN}]}
					F=-1
				else
					F=0
					echo Child $CN on ${host[${CN}]} Running!
				fi
			fi
			(( CN = CN + 1 ))
		done
	done
	
	if [ $F -lt "0" ]; # error occur!
	then
		exit 1
	else
		# Do normalization
		newTable=${TMPDIR}/HMM_R_${CUR}_TTABLE.tbl
		CONF=/tmp/${CUR}_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo cm1 ${CUR}>> $CONF
		echo outputttable ${newTable}>>$CONF
		if [ $CUR = $TOTALROUND ];
		then
			echo actmodel1 0 >> $CONF
		else
			echo actmodel1 0 >> $CONF # if we want actual table, then do it here
		fi
		echo prevtmodel ${prev}>>$CONF
		cml=`echo ${MODEL_ONE_NORM} ${CONF}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${output[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo $cml
		if ${cml}>>${LOGFILE};
		then	
			echo " Combine t model ok"
		else
			echo " Combination failed on t model!"
			exit 1
		fi
		
		newATable=${TMPDIR}/HMM_R_${CUR}_ATABLE.tbl
		CONF=/tmp/${CUR}_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo cm2 ${CUR}>> $CONF
		echo outputatable ${newATable}>>$CONF
		if [ $CUR = $TOTALROUND ];
		then
			echo actmodel1 0 >> $CONF
		else	
			echo actmodel1 0 >> $CONF # if we want actual table, then do it here
		fi
		if [ $CUR -gt "1" ]; 
		then
			echo prevamodel ${prev_a}>>$CONF
		fi
		cml=`echo ${ATABLE_EXEC} ${CONF}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${acount[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo $cml
		if ${cml}>>${LOGFILE};
		then
			echo " Combine a model ok"
		else
			echo " Combination failed on a model!"
			exit 1
		fi
		
		newHTable=${TMPDIR}/HMM_R_${CUR}_HTABLE.tbl
		CONF=/tmp/${CUR}_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo cm2 ${CUR}>> $CONF
		echo outputhtable ${newHTable}>>$CONF
		if [ $CUR = $TOTALROUND ];
		then
			echo actmodel1 0 >> $CONF
		else	
			echo actmodel1 0 >> $CONF # if we want actual table, then do it here
		fi
		cml=`echo ${CHMM_EXEC} ${CONF}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${hcount[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo $cml
		if ${cml}>>${LOGFILE};
		then
			echo " Combine hmm model ok"
		else
			echo " Combination failed on hmm model!"
			exit 1
		fi		
	fi
	(( CUR = CUR + 1 ))	
done

if [ -e ${newTable} ];
then
	if [ -e ${newATable} ]
	then
		echo "No more iteration"
		echo "Copying last round's table to " $OUTPUT
		cp -f ${newATable} ${OUTPUT_A}
		cp -f ${newTable} ${OUTPUT}
		if [ -e $newHTable ] && [ -e ${newHTable}.alpha ]  && [ -e ${newHTable}.beta ];
		then
			cp -f $newHTable ${OUTPUT_H} 
			cp -f ${newHTable}.alpha ${OUTPUT_H}.alpha
			cp -f ${newHTable}.beta ${OUTPUT_H}.beta
		fi
	fi
fi



