#!/usr/bin/env bash

# This is the master of model one training
# Parameters needed:
echo "--------------------------------------------------------"
echo "Usage of masters"
echo "modelOneMaster <template> <watchdir> <collectdir> <logdir> <output> <round> <tmp> <childlist>"
echo "<template> The template file for the training"
echo "<watchdir> The directory to watch the output of childs"
echo "<collectdir> The directory to collect output of childs"
echo "<logdir> The directory to collect log of childs"
echo "<round> The total number of iterations"
echo "<tmp> the temporary dir (Child process should be able to access)"
echo "<childlist> is the list of childs"
echo "--------------------------------------------------------"
echo 

if [ -e $PWD/defines.inc ];
then
        source $PWD/defines.inc
else
	MODEL_ONE_NORM=/SMT/giza/view_source/newgiza/src/para/renomalizeModel1
fi

if [ $# -lt "8" ];
then
	echo " NO! I need your command!"
	exit 1
fi

echo "... Checking template" $1
if [ -r $1 ]; # is template exists?
then
	echo " OK"
else
	echo " NO! The template cannot be read! "
	exit 1
fi

echo "... Checking watching dir" $2
if [ -d $2 ]; # is watch dir exists
then
	echo " OK"
	rm -f $2/*
else
	echo " NO! The watching dir cannot be accessed"
	exit 1
fi

echo "... Checking collecting dir" $3
if [ -d $3 ]; # is collection OK?
then
	echo " OK"
else
	echo " NO! The collecting dir cannot be accessed"
	exit 1
fi

echo "... Checking log dir" $4
if [ -d $4 ]; # is log OK?
then
	echo " OK"
else
	echo " NO! The log dir cannot be accessed"
	exit 1
fi

echo "... Checking temporary dir"
if [ -d $7 ]; # is tmp OK?
then
	echo " OK"
else
	echo " NO! The temporary dir cannot be accessed"
	exit 1
fi

echo "... Checking child list"
if [ -r $8 ]; # is childlist OK?
then
	echo " OK"
else
	echo " NO! The temporary dir cannot be accessed"
	exit 1
fi

TEMPL=$1
WATCHDIR=$2
COLLDIR=$3
LOGDIR=$4
TMPDIR=$7
CHILDLIST=$8

LOGFILE=${LOGDIR}/model1_master.log.$$

touch $5

if [ -w $5 ]; #is output writable?
then
	echo "... Checking access to output file"
	echo " OK"
else
	echo " NO! I will not be able to write output file"
	exit 1
fi

OUTPUT=$5

if [ $6 -lt 1 ];
then
	echo " At least one round of training should be taken "
	exit 1
fi

ROUNDS=$6
TOTALROUND=$6
((ROUNDS = ROUNDS + 1))

echo "We will do " $TOTALROUND " (minus one) rounds of training"


# If we survives here, let's start running

CUR=1 # Current round
#Total number of childs

TOTAL_CHILD=`wc -l $CHILDLIST | awk '{print $1}' `

echo "Total number of children : " $TOTAL_CHILD

((TOTAL_CHILD = TOTAL_CHILD + 1))
CN=1
while [ $CN -lt $TOTAL_CHILD ];
do
	host[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $1}'`
	exe[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $2}'`
	conf[${CN}]=`head $CHILDLIST -n $CN | tail -n 1 | awk '{print $3}'`
	(( CN = CN + 1 ))
done


while [ $CUR -lt $ROUNDS ];
do
	echo "Start running round " $CUR
	if [ $CUR -gt "1" ]; # not first round,
	then
		(( PP = CUR - 1 ))
		prev=${TMPDIR}/R_${PP}_TTABLE.tbl
	fi
	#################################################
	# Start all the process
	#################################################
	CN=1
	rm -f $2/*
	while [ $CN -lt $TOTAL_CHILD ];
	do
		output[${CN}]=$COLLDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.diff
		signal[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish
		start[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish.starting
		fail[${CN}]=$WATCHDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.finish.failed
		log[${CN}]=$LOGDIR/R_${CUR}_P_${CN}_AT_${host[${CN}]}.log

		echo "cd" $PWD > ${CN}.tmp
		echo ${exe[${CN}]} ${conf[${CN}]} ${output[${CN}]} ${CUR} ${signal[${CN}]} ${log[${CN}]} $prev >> ${CN}.tmp
		
		command[${CN}]=`echo ssh ${host[$CN]} source $PWD/${CN}.tmp `

#rm -f ${CN}.tmp
		echo ${command[${CN}]}
                touch  ${start[${CN}]}	
            ${command[${CN}]} >> ${LOGFILE}&
#	    exit 1
            echo Child ${CN} Started on ${host[$CN]}
            (( CN = CN + 1 ))
	done
	################################################
	# Watch what's going on
	################################################
	F=0
	while [ $F = "0" ]
	do
		sleep 20
		CN=1
		F=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			if [ -e ${start[${CN}]} ]; # Starting process failed
			then
				echo Starting child $CN failed!
				echo  ${host[${CN}]}  ${exe[${CN}]}  ${conf[${CN}]}
			fi	
			if [ -e ${signal[${CN}]} ]; # Finished
			then
				echo Child $CN on ${host[${CN}]} completed
			else
				if [ -e ${fail[${CN}]} ];
				then
					echo Child $CN on ${host[${CN}]} FAILED!
					echo Stop process and check ${log[${CN}]}
					F=-1
				else
					F=0
					echo Child $CN on ${host[${CN}]} Running!
				fi
			fi
			(( CN = CN + 1 ))
		done
	done
	
	if [ $F -lt "0" ]; # error occur!
	then
		exit 1
	else
		# Do normalization
		newTable=${TMPDIR}/R_${CUR}_TTABLE.tbl
		CONF=/tmp/${CUR}_NORMAL.gizacfg
		rm -f $CONF
		cat $TEMPL > $CONF
		echo cm1 ${CUR}>> $CONF
		echo outputttable ${newTable}>>$CONF
		if [ $CUR = $TOTALROUND ];
		then
			echo actmodel1 0 >> $CONF
		else
			echo actmodel1 0 >> $CONF # if we want actual table, then do it here
		fi
		if [ $CUR -gt "1" ]; 
		then
			echo prevtmodel ${prev}>>$CONF
		fi
		cml=`echo ${MODEL_ONE_NORM} ${CONF}`
		CN=1
		while [ $CN -lt $TOTAL_CHILD ];
		do
			cml=`echo ${cml}  ${output[${CN}]} | tr -s / /`
			(( CN = CN + 1 ))
		done
		echo $cml
		if ${cml}>>${LOGFILE};
		then	
			echo " Starting next step!"
		else
			echo " Combination failed!"
			exit 1
		fi
	fi
	(( CUR = CUR + 1 ))	
done

if [ -e ${newTable} ];
then
	echo "No more iteration"
	echo "Copying last round's table to " $OUTPUT
	cp -f ${newTable} ${OUTPUT}
fi



