#!/usr/bin/env bash

# First of all, make sentence


echo You should provide :
echo "Arg1 : The source side sentences (file name) here" $1
echo "Arg2 : The target side sentences (file name) here" $2
echo "Arg3 : What is the template of all the sub" $3
echo "Arg4 : The machine pool " $4


if [ -e $PWD/defines.inc ];
then
        source $PWD/defines.inc
else
	echo "You should provide defines.inc at current directory"
	exit 1
fi

if [ $# -lt 4 ];
then
 	echo No enough arguments
	exit 1
fi

SOURCE_RAW=data/$1
TARGET_RAW=data/$2
TEMPL_RAW=$3

if [ -e ${TEMPL_RAW} ];
then
	TEMPL=${TEMPL_RAW}
	echo "Template check OK" ${TEMPL_RAW}
else
	echo "Template not fould" ${TEMPL_RAW}
	exit 1
fi

if [ -e ${SOURCE_RAW} ];
then 
	SOURCE_LEN=`wc -l ${SOURCE_RAW} | awk '{print $1}'`
	echo Length of source corpora : ${SOURCE_LEN}
else
	echo "FAILED, CANNOT FIND SOURCE CORPORA"
	exit 1
fi

if [ -e ${TARGET_RAW} ];
then 
	TARGET_LEN=`wc -l ${TARGET_RAW} | awk '{print $1}'`
	echo Length of target corpora : ${TARGET_LEN}
else
	echo "FAILED, CANNOT FIND TARGET CORPORA"
	exit 1
fi

if [ $SOURCE_LEN != $TARGET_LEN ];
then
	echo "THE SOURCE AND TARGET MISMATCHED"
	exit 1
fi


${BIN_DIR}/plain2snt.out ${SOURCE_RAW} ${TARGET_RAW}
source=`basename $1`
target=`basename $2`
mv data/*.snt data/*.vcb snt_vcb
w=$PWD/snt_vcb
snt_file=snt_vcb/${source}_${target}.snt
${BIN_DIR}/snt2plain.out $w/${source}.vcb $w/${target}.vcb ${snt_file} PLAIN
rm -f $w/mkcls.log
echo "Making word classes"
${BIN_DIR}/mkcls -m2 -pPLAIN1.txt -c50 -V$w/${source}.vcb.classes opt >& $w/mkcls1.log
rm PLAIN1.txt
echo "Making word classes"
${BIN_DIR}/mkcls -m2 -pPLAIN2.txt -c50 -V$w/${target}.vcb.classes opt >& $w/mkcls2.log
rm PLAIN2.txt

echo We are trying to get profile of the corpora, please wait...
PROF=`cat ${snt_file} | awk 'BEGIN{CNT=0;SUM=0;N=0;}{if(N % 3 == 0){SUM+=$1;CNT+=1;} N++;}END{print CNT " "  SUM }'`
sentPair=`echo $PROF | awk '{print $1}'`
sentPair2=`echo $PROF | awk '{print $2}'`

echo "Sentence Count     : " $sentPair
echo "Sentence Weight Sum: " ${sentPair2}

MC_FILE=$4
V=`wc -l ${MC_FILE} | awk '{print $1}'`

(( PER_FILE = SOURCE_LEN / V + 1 ))

echo You will have $PER_FILE lines per file, totally ${V} files.

(( APERFILE = PER_FILE * 3 ))

split -d --lines=${APERFILE} $snt_file ${snt_file}_split_

rm -f split/*

mv snt_vcb/*split_?? split

echo Calling ${SCRIPT_DIR}/generate_template.sh $w/${source}.vcb $w/${target}.vcb ${TEMPL_RAW} ${MC_FILE} $sentPair ${sentPair2}
${SCRIPT_DIR}/generate_template.sh $w/${source}.vcb $w/${target}.vcb ${TEMPL_RAW} ${MC_FILE} $sentPair ${sentPair2}

echo Generation OK, start Training!


# OK, Start Model One

#source $PWD/defines.inc

CML_ONE=`echo ${MODEL_ONE_MASTER} ${GENERAL_TMPL} ${WATCH_DIR} ${COLL_DIR} ${LOG_DIR} ${OUTPUT_MODEL_ONE} ${MODEL_ONE_ROUND} ${TMP_DIR} ${CHILD_LIST_ONE}`

echo "Invoking model one master: "
echo ${CML_ONE}
echo "----------MODEL_ONE_AFTER_BY------------"

if ${CML_ONE} ;
then
	if [ -e ${OUTPUT_MODEL_ONE} ];
	then
		echo "Model One Training Completed"
		echo "Continue..."
	else
		echo "MODEL ONE DID NOT OUTPUT CORRECT FILES"
		echo "They are : TTable file" ${OUTPUT_MODEL_ONE}
		echo "QUITTING"
		exit 1
	fi
else
	echo "MODEL ONE TRAINING FAILED!"
	echo "QUITTING"
	echo "-------------------------------------"
	exit 1
fi

# Now start hmm training
# Here is all the variables we needed


CML_HMM=`echo ${MODEL_HMM_MASTER} ${GENERAL_TMPL} ${WATCH_DIR} ${COLL_DIR} ${LOG_DIR} ${OUTPUT_T_HMM} ${OUTPUT_A_HMM} ${OUTPUT_H_HMM} ${MODEL_HMM_ROUND} ${TMP_DIR} ${CHILD_LIST_HMM} ${INPUT_T_HMM}`

echo "Invoking HMM master: "
echo ${CML_HMM}
echo "----------MODEL_HMM_AFTER_BY------------"

if ${CML_HMM} ;
then
	if [ -e ${OUTPUT_T_HMM} ];
	then
		echo "T Table OK Continue..."
	else
		echo "MODEL HMM DID NOT OUTPUT CORRECT FILES"
		echo "They are : TTable file" ${OUTPUT_T_HMM}
		echo "QUITTING"
		exit 1
	fi
	if [ -e ${OUTPUT_A_HMM} ];
	then
		echo "A Table OK Continue..."
	else
		echo "MODEL HMM DID NOT OUTPUT CORRECT FILES"
		echo "They are : ATable file" ${OUTPUT_A_HMM}
		echo "QUITTING"
		exit 1
	fi
	if [ -e ${OUTPUT_H_HMM} ];
	then
		echo "H Table OK Continue..."
	else
		echo "MODEL HMM DID NOT OUTPUT CORRECT FILES"
		echo "They are : HTable file" ${OUTPUT_H_HMM}
		echo "QUITTING"
		exit 1
	fi
else
	echo "MODEL HMM TRAINING FAILED!"
	echo "QUITTING"
	echo "-------------------------------------"
	exit 1
fi

# Now start 345 training
# Here is all the variables we needed


CML_345=`echo ${MODEL_345_MASTER} ${GENERAL_TMPL} ${WATCH_DIR} ${COLL_DIR} ${LOG_DIR} ${OUTPUT_T_FINAL} ${OUTPUT_A_FINAL} ${OUTPUT_D_FINAL} ${OUTPUT_N_FINAL} ${OUTPUT_PZ_FINAL} ${MODEL_345_ROUND} ${TMP_DIR} ${CHILD_LIST_345} ${INPUT_T_FINAL} ${INPUT_A_FINAL} ${INPUT_H_FINAL}`

echo "Invoking Model 3/4/5 master: "
echo ${CML_345}
echo "----------MODEL_345_AFTER_BY------------"

if ${CML_345} ;
then
	if [ -e ${OUTPUT_T_FINAL} ];
	then
		echo "T Table OK Continue..."
	else
		echo "MODEL 345 DID NOT OUTPUT CORRECT FILES"
		echo "They are : TTable file" ${OUTPUT_T_FINAL}
		echo "QUITTING"
		exit 1
	fi
	if [ -e ${OUTPUT_A_FINAL} ];
	then
		echo "A Table OK Continue..."
	else
		echo "MODEL 345 DID NOT OUTPUT CORRECT FILES"
		echo "They are : ATable file" ${OUTPUT_A_FINAL}
		echo "QUITTING"
		exit 1
	fi
	if [ -e ${OUTPUT_N_FINAL} ];
	then
		echo "N Table OK Continue..."
	else
		echo "MODEL 345 DID NOT OUTPUT CORRECT FILES"
		echo "They are : NTable file" ${OUTPUT_N_FINAL}
		echo "QUITTING"
		exit 1
	fi
	if [ -e ${OUTPUT_D_FINAL} ];
	then
		echo "D Table OK Continue..."
	else
		echo "MODEL 345 DID NOT OUTPUT CORRECT FILES"
		echo "They are : DTable file" ${OUTPUT_D_FINAL}
		echo "QUITTING"
		exit 1
	fi
	if [ -e ${OUTPUT_PZ_FINAL} ];
	then
		echo "P0 Table OK Continue..."
	else
		echo "MODEL 345 DID NOT OUTPUT CORRECT FILES"
		echo "They are : P0Table file" ${OUTPUT_PZ_FINAL}
		echo "QUITTING"
		exit 1
	fi
else
	echo "MODEL 345 TRAINING FAILED!"
	echo "QUITTING"
	echo "-------------------------------------"
	exit 1
fi

echo "Well, everything is done"


